<?php

namespace App\Factory;

use App\Exception;

class PaymentFactory {

    private $connection;
    private $util;

    public function __construct (\App\Util\Util $util,
                                 \Doctrine\DBAL\Connection $connection) {
        
        $this->util = $util;
        $this->connection = $connection;
    }

    public function getPaymentCode ()
    {
        $sql = "
            SELECT TP.CDTIPORECE 
              FROM TIPORECEINTE TP
              JOIN INTEGRACAO IT
                ON IT.CDINTEGRACAO = TP.CDINTEGRACAO
             WHERE IT.IDINTEGRACAO = 'ANT'
        ";

        return $this->connection->fetchAssoc($sql);
    }

    public function getMoneyReceiptCode ()
    {
        $sql = "SELECT MIN(CDTIPORECE) AS CDTIPORECE
                FROM TIPORECE
                WHERE IDTIPORECE = '4'";
        
        $result = $this->connection->fetchAll($sql);
        return $result[0]["CDTIPORECE"];
    }

    public function saveMovcaixadlv ($idtipomovivedlv, $cdfilial, $nrvendarest, $nrseqmovdlv, $paymentCode, $paymentValue, $cdcliente, $cdconsumidor)
    {
        
        $sql = "
            INSERT INTO MOVCAIXADLV
        		(CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
        		VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR)
        	VALUES
        		(:CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
        		:VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR)";

        $params = [
            "CDFILIAL"        => $cdfilial,
	        "NRVENDAREST"     => $nrvendarest,
	        "NRSEQMOVDLV"     => $nrseqmovdlv,
	        "IDTIPOMOVIVEDLV" => $idtipomovivedlv,
	        "CDCLIENTE"       => $cdcliente,
	        "CDCONSUMIDOR"    => $cdconsumidor,
	        "CDTIPORECE"      => $paymentCode,
	        "VRMOVIVENDDLV"   => $this->util->tofloat($paymentValue)
        ];

        return $this->connection->executeQuery($sql, $params);
    }
    
    public function getReceiptType ($receipt, $cdFilial)
    {
        //===================== Busca por filial =====================
        $sql = "
            SELECT CDTIPORECE
              
              FROM TIPORECEINTELOCAL TP
              
              JOIN INTEGRACAO INTE 
                ON TP.CDINTEGRACAO   = INTE.CDINTEGRACAO
             
             WHERE INTE.IDINTEGRACAO = 'ANT' 
               AND CDFILIAL          = :CDFILIAL
               AND UPPER(REPLACE(CDTPRECEINTE, ' ', '')) = :CDTPRECEINTE    
        ";

        $params = [
            ":CDFILIAL"     => $cdFilial,
            ":CDTPRECEINTE" => mb_strtoupper(preg_replace("/\s+/", "", $receipt))
        ];

        $result = $this->connection->fetchAll($sql, $params);
        
        if(!empty($result) && isset($result[0]["CDTIPORECE"])) 
            return $result[0]["CDTIPORECE"];
        
        //===================== Busca pela rede =====================

        $sql = "
            SELECT CDTIPORECE

              FROM TIPORECEINTE TP
            
              JOIN INTEGRACAO INTE 
                ON TP.CDINTEGRACAO   = INTE.CDINTEGRACAO

             WHERE INTE.IDINTEGRACAO = 'ANT' 
               AND UPPER(REPLACE(CDTPRECEINTE, ' ', '')) = :CDTPRECEINTE
        ";
        
        $params = [
            ":CDTPRECEINTE" => mb_strtoupper(preg_replace("/\s+/", "", $receipt))
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        
        if(!empty($result)) {
            return $result[0]["CDTIPORECE"];
        } else {
            return $result;
        }
    }
}
