<?php

namespace App\Factory;

use App\Exception\SqlException as SqlException;
use App\Exception\ParamsException as ParamsException;

class ProductFactory {

    private $connection;
    private $util;

    public function __construct (\App\Util\Util $util,
            \Doctrine\DBAL\Connection $connection) {
        $this->util = $util;
        $this->connection = $connection;
    }

    public function getObservation ($observationCode)
    {

        $obs  = explode("|", $observationCode);

        $obs0 = preg_match("/[0-9a-zA-Z]{2}/", $obs[0]);
    	$obs1 = preg_match("/[0-9a-zA-Z]{2}/", $obs[1]);
    	
        if(strlen($obs[0]) != 2 || !$obs0)
    	    throw new ParamsException("
    	        O Código Grupo de Observação ({$obs[0]}) no código externo {$observationCode} deve possuir dois caracteres alfanuméricos, sem caracteres especiais.");
    	
    	if(strlen($obs[1]) != 2 || !$obs1)
    	    throw new ParamsException("
    	        O Código de Observação ({$obs[1]}), no código externo {$observationCode} deve possuir dois caracteres alfanuméricos, sem caracteres especiais.");
    	
        //===================== Busca por CDGRPOCOR =====================
    	
    	$sql = "
    		SELECT *
    		  FROM OCORRENCIA
    		 WHERE CDGRPOCOR = :CDGRPOCOR";

    	$params = array(
    		":CDGRPOCOR" => $obs[0]
    	);
    	
    	$cdgrpocor = $this->connection->fetchAssoc($sql, $params);
    	
    	if(!$cdgrpocor)
    	     throw new ParamsException("
    	        O código externo {$observationCode} não foi encontrado.<br>
    	        Verifique no Odhen Retail o código externo desta observação e faça a correção no portal Anota Aí.");

    	//===================== Busca por CDOCOR =====================
    	
    	$sql = "
    		SELECT *
    		   FROM OCORRENCIA
    		  WHERE CDGRPOCOR = :CDGRPOCOR
    		    AND CDOCORR = :CDOCORR";

    	$params = array(
    	    ":CDGRPOCOR" => $obs[0],
    		  ":CDOCORR" => $obs[1]
    	);
    	
    	$cdocor = $this->connection->fetchAssoc($sql, $params);
    	if(!$cdocor)
    	     throw new ParamsException("
    	        O código externo {$observationCode} não foi encontrado.<br>
    	        Verifique no Odhen Retail o código externo desta observação e faça a correção no portal Anota Aí.");
        
        return $cdocor;
    }
    
    public function getProduct ($product)
    {
        $sql = "
            SELECT CDPRODUTO, CDARVPROD, NMPRODUTO, IDTIPOCOMPPROD
              FROM PRODUTO PR
             WHERE CDARVPROD  = :CDARVPROD
                OR CDPRODINTE = :CDPRODINTE
                OR CDPRODUTO  = :CDPRODUTO
        ";

        $productCode = str_replace(".", "", $product);
        $productCode = preg_replace("/[^0-9A-Z]/", "", $productCode);

        $params = array(
            ":CDARVPROD"  => $productCode,
            ':CDPRODINTE' => $productCode,
            ':CDPRODUTO'  => $productCode
        );

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function insertProduct ($product)
    {
        
        $sql = "
            INSERT INTO ITCOMANDAVEN
    			(CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN,
    			CDPRODUTO, QTPRODCOMVEN, VRPRECCOMVEN, TXPRODCOMVEN,
    			IDSTPRCOMVEN, VRDESCCOMVEN, NRLUGARMESA, DTHRINCOMVEN,
    			IDPRODIMPFIS, CDLOJA, VRACRCOMVEN, NRSEQPRODCOM, NRSEQPRODCUP,
    			VRPRECCLCOMVEN, CDCAIXACOLETOR, CDPRODPROMOCAO, CDVENDEDOR, NRINSCRCONS, DSOBSPEDDIGCMD)
    		VALUES
    			(:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN,
    			:CDPRODUTO, :QTPRODCOMVEN, :VRPRECCOMVEN, :TXPRODCOMVEN,
    			:IDSTPRCOMVEN, :VRDESCCOMVEN, :NRLUGARMESA, TO_DATE(:CURRDATE, 'dd/mm/yyyy hh24:mi:ss'),
    			:IDPRODIMPFIS, :CDLOJA, :VRACRCOMVEN, :NRSEQPRODCOM, :NRSEQPRODCUP,
    			:VRPRECCLCOMVEN, :CDCAIXACOLETOR, :CDPRODPROMOCAO, :CDVENDEDOR, :NRINSCRCONS, :DSOBSPEDDIGCMD)";
    		
	    date_default_timezone_set($this->util->getCurrentTimezone(strtoupper($product["SGESTADO"])));
	    
    	$params = array(
    	    ":CDFILIAL"       => $product["CDFILIAL"],
			":NRVENDAREST"    => $product["NRVENDAREST"],
			":NRCOMANDA"      => $product["NRCOMANDA"],
			":NRPRODCOMVEN"   => $product["NRPRODCOMVEN"],
			":CDPRODUTO"      => preg_replace("/[^0-9A-Z]/", "", $product["CDPRODUTO"]),
			":QTPRODCOMVEN"   => $product["QTPRODCOMVEN"],
			":VRPRECCOMVEN"   => $this->util->toFloat($product["VRPRECCOMVEN"]),
			":TXPRODCOMVEN"   => $product["TXPRODCOMVEN"],
			":IDSTPRCOMVEN"   => $product["IDSTPRCOMVEN"],
			":VRDESCCOMVEN"   => $this->util->toFloat($product["VRDESCCOMVEN"]),
			":NRLUGARMESA"    => $product["NRLUGARMESA"],
			":CURRDATE"       => date("d/m/Y H:i:s"),
			":IDPRODIMPFIS"   => $product["IDPRODIMPFIS"],
			":CDLOJA"         => $product["CDLOJA"],
			":VRACRCOMVEN"    => $this->util->toFloat($product["VRACRCOMVEN"]),
			":NRSEQPRODCOM"   => $product["NRSEQPRODCOM"],
			":NRSEQPRODCUP"   => $product["NRSEQPRODCUP"],
			":VRPRECCLCOMVEN" => $this->util->toFloat($product["VRPRECCLCOMVEN"]),
			":CDCAIXACOLETOR" => $product["CDCAIXACOLETOR"],
			":CDPRODPROMOCAO" => preg_replace("/[^0-9A-Z]/", "", $product["CDPRODPROMOCAO"]),
			":CDVENDEDOR"     => $product["CDVENDEDOR"],
			":NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $product["NRINSCRCONS"]),
			":DSOBSPEDDIGCMD" => $product["TXPRODCOMVEN"]
    	);
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertObservation ($obs)
    {
        $sql = "
            INSERT INTO	OBSITCOMANDAVEN
                (CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN, CDOCORR, CDGRPOCOR, NRPRODCOMVENOBS, QTPRODCOMVENOBS)
            VALUES
                (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN, :CDOCORR, :CDGRPOCOR, :NRPRODCOMVENOBS, :QTPRODCOMVENOBS)";
		
	    $params = [
	        ":CDFILIAL"        => $obs["CDFILIAL"],
            ":NRVENDAREST"     => $obs["NRVENDAREST"],
            ":NRCOMANDA"       => $obs["NRCOMANDA"],
            ":NRPRODCOMVEN"    => $obs["NRPRODCOMVEN"],
            ":NRPRODCOMVENOBS" => $obs["NRPRODCOMVENOBS"],
            ":CDOCORR"         => $obs["CDOCORR"],
            ":CDGRPOCOR"       => $obs["CDGRPOCOR"],
            ":QTPRODCOMVENOBS" => $obs["QTPRODCOMVENOBS"]  
        ];

        $this->connection->executeQuery($sql, $params);
    }
}
