<?php
namespace App\Controller;

use App\Exception\SqlException as SqlException;
use App\Util\Util;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class WebhookController {  

    private $connection;
    private $webhookService;
    private $util;

    const API_ANT_BASE_URL  = "https://api-parceiros.anota.ai/";

    public function __construct(\Doctrine\DBAL\Connection $connection,
                                \App\Service\WebhookService $webhookService,
                                \App\Util\Util $util) {

        $this->connection       = $connection;
        $this->webhookService   = $webhookService;
        $this->util             = $util;
    }

    public function welcome(Request $request, Response $response) {
        return $response->withJson([
            'success' => true,
            'message' => "It works",
            'connection' => $this->connection->ping(),
            'data'    => []
        ]);
    }

    public function order(Request $request, Response $response) {
        $response->withStatus(200);
        
        try {
            $input = file_get_contents('php://input');

            if(empty($input)) 
                throw new \Exception("Não foram encontrados parâmetros de entrada!");
            
            $body  = json_decode($input, true);

            if(!empty($body["canceled"]) && $body["canceled"] == true) {
                $route = "/cancelOrder";
            } else {
                $route = "/order";
            }

            $token = $request->getHeaderLine('Authorization');
            $clientData = $this->webhookService->getClientData($token);
            
            if(empty($clientData)) {
                $this->util->writeLog('events', (date("d-m-Y H-i-s") . ' ' . $body['merchant']['id'] . '.txt'), $input);
                throw new \Exception("Nenhum clinte associado ao token $token");
            }
            
            $this->util->writeLog('events', (date("d-m-Y H-i-s") . ' ' . $body['merchant']['id'] . '.txt'), $input, $clientData['DBNAME']);

            $url        = $clientData['URL'] . $route;
            $clientName = $clientData['NOME'];
            $clientType = $clientData['TIPO'];
            $dbName     = $clientData['DBNAME'];
                
            $header = [
                "Content-type: application/json",
                "x-client-name: $clientName"
            ];
            
            if($clientType == 'hosting' || $clientType == 'saas'){
                $header = [
                    "Content-type: application/json",
                    "x-client-name: $clientName",
                    "x-client-type: $clientType",
                    "x-client-env: $dbName"
                ];
            }

            $requestContent = [
                'auth'     => '81743A8E8D41AF82A4807097CDF4BEC2DDDB9E6334BECCFED4D46FC783A8CA2E',
                'anotaAiData' => $body
            ];
            
            $reqClient = json_encode($this->util->request($url, json_encode($requestContent), $header), true);
            
            if(isset($reqClient['error']) && $reqClient['error']) {
              throw new \Exception($reqClient['message']);
            }

        } catch(\Exception $e) {
          $clientName = isset($dbName) ? $dbName : '';
          $message = "Erro no envio do pedido. " . $e->getMessage();
          $this->util->writeLog('error', (date("d-m-Y H-i-s") . ' - ' . $body['merchant']['id'] . '.txt'), $message, $clientName);
        }
    }

    private function debugLog($params){
        $folder      = "/home/developer/workfolder/logs/anotaAi/debugWH";
        $fileName = round(microtime(true)*1000).'_order';
        $currentFile = "$folder/$fileName";
        if(!file_exists($folder)) {
            mkdir($folder, 0775, true);
        }
        // file_put_contents($currentFile, json_encode($params, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        file_put_contents($currentFile, $params);
    }

    //run at every monday at 9hs | Cron: 0 9 * * 1 
    public function databaseBackup(Request $request, Response $response) {
        $body = file_get_contents(__DIR__ . "/../../templates/defaultMail.html");
        
        $this->util->sendDatabase("Anota-ai Webhook", $body, [
            ":DATA_ATUAL"    => date("d/m/Y"),
            ":GERADO_EM"     => date("d/m/Y H:i:s")
        ]);

        return $response->withJson([
            'success' => true,
            'message' => "",
            'data'    => []
        ]);
    }

    public function getClientList(Request $request, Response $response) {
        try {
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->getClientList()
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function addClient(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->addClient($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function editClient(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->editClient($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function deleteClient(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "Cliente deletado com sucesso!",
                'data'    => $this->webhookService->deleteClient($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function addToken(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->addToken($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function editToken(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->editToken($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function deleteToken(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "Token deletado com sucesso!",
                'data'    => $this->webhookService->deleteToken($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
     public function cleanToken(Request $request, Response $response) {
        try {
            return $response->withJson([
                'success' => true,
                'message' => "Token removido com sucesso!",
                'data'    => $this->webhookService->cleanToken()
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
    public function renovateToken(Request $request, Response $response) {
        try {
            return $response->withJson([
                'success' => true,
                'message' => "Token renovado com sucesso!",
                'data'    => $this->webhookService->getAuthTokens()
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
}